﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace ControlInvoke {
  public partial class Form1 : Form {
    public Form1( ) {
      InitializeComponent( );
    }

    Thread thread;

    private void button1_Click(object sender, EventArgs e) {
      if (thread != null) {
        MessageBox.Show("Thread already running");
        return;
      }

      thread = new Thread(new ThreadStart(ThreadMethod));
      thread.IsBackground = true;
      thread.Start( );
    }

    private void button2_Click(object sender, EventArgs e) {
      if (thread != null) {
        thread.Abort( );
        thread = null;
      }
    }

    void ThreadMethod( ) {
      int value = 0;

      try {
        while (true) {
          UpdateValueInUI(value++);
          Thread.Sleep(200);
        }
      }
      catch (ThreadAbortException) {

      }
    }

    // This doesn't work on its own in .NET 2, because a cross-thread call
    // exception is thrown. In .NET 1 we don't have that exception, but that
    // doesn't make such a direct call any better - you won't get the exception
    // but the application will still crash sooner or later.
    //
        private void UpdateValueInUI(int value) {
          statusLabel.Text = value.ToString( );
        }


    // This is what the solution would have looked like in .NET 1
    //
//        delegate void UpdateValueInUIHandler(int value);
//    
//        private void UpdateValueInUI(int value) {
//          if (statusLabel.InvokeRequired)
//            statusLabel.Invoke(new UpdateValueInUIHandler(UpdateValueInUI), new object[] { value });
//          else
//            statusLabel.Text = value.ToString( );
//        }

    // In .NET 2 we have anonymous methods, so we can make it a bit nicer

//        private void UpdateValueInUI(int value) {
//          if (statusLabel.InvokeRequired)
//            statusLabel.Invoke((MethodInvoker) delegate {
//              UpdateValueInUI(value);
//            });
//          else
//            statusLabel.Text = value.ToString( );
//        }

    // Finally in .NET 3 things get really nice, readable and easy to use with the
    // help of extension methods and lambdas. 

    //private void UpdateValueInUI(int value) {
    //  if (statusLabel.Invoke(( ) => UpdateValueInUI(value)))
    //    statusLabel.Text = value.ToString( );
    //}
  }

  //public static class InvokeHelpers {
  //  public static bool Invoke(this Control control, MethodInvoker invoker) {
  //    if (control.InvokeRequired) {
  //      control.Invoke(invoker);
  //      return false;
  //    }
  //    return true;
  //  }
  //}
}