﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Query;
using System.Xml.XLinq;
using System.Data.DLinq;
using System.IO;
using System.Reflection;

namespace LINQToSQL
{
    class Program
    {
        static void Main(string[] args)
        {
            //DataContextWithAttributedClass();
            //StonglyTypedDataContext();
            //!!WE NEED TO COMMENT IN THE MAPPED CUSTOMER AND ORDER HERE!!
            //PersistanceIgnoranceApproach();
            //NavigateRelationships();
            //DemonstrateIdentityMap();
            //AdhocJoins();
            //WorkingWithHeirachicalData();
            //DeferredExecution();
            //LazyLoading();
            //AddAndDeleteRows();
            //Update();
            //FullGenDemo.Demo();

        }

  
        #region 1: Uses a generic data context and a class marked with attributes
        private static void DataContextWithAttributedClass()
        {
            DataContext session = new DataContext("Server=.;Database=Northwind;Trusted_Connection=True;");

            Table<Customer> Customers = session.GetTable<Customer>();

            var query =
                from c in Customers
 			            where c.City == "London"
 			            select c;

            ObjectDumper.Write(query);
        }
        #endregion

        #region 2: We can use a strongly typed data context to save on those GetTable calls
        private static void StonglyTypedDataContext()
        {
            Northwind session = new Northwind("Server=.;Database=Northwind;Trusted_Connection=True;");

            var query =
                from c in session.Customers
                where c.City == "London"
                select c;

            ObjectDumper.Write(query);
        }
        #endregion

        #region 3: We can use a mapping file so as to get a persistance ignorance approach
        private static void PersistanceIgnoranceApproach()
        {
            //Need to comment out the attributed version of the Customer mapping so we can prove this really works
            //It will work with the attributed version but the mapping overrides (a mistake proves this)
            Northwind session = new Northwind("Server=.;Database=Northwind;Trusted_Connection=True;", Mapping.GetMapping());

            var query =
                from c in session.Customers
		                where c.City == "London"
		                select c;

            ObjectDumper.Write(query);
        }
        #endregion

        #region 4: We can navigate relationships between tables
        private static void NavigateRelationships()
        {
            //Need to comment out the attributed version of the Customer mapping so we can prove this really works
            //It will work with the attributed version but the mapping overrides (a mistake proves this)
            NorthwindContext session = new NorthwindContext("Server=.;Database=Northwind;Trusted_Connection=True;", Mapping.GetMapping());
            session.Log = Console.Out;

            var query =
                    from c in session.Customers
		                     where c.City == "London"
		                     from o in c.Orders
		                      where o.OrderDate.Value.Year == 1997
		                      select new {c.CompanyName, o.OrderID, o.OrderDate};

            ObjectDumper.Write(query);
        }
        #endregion

        #region 5: Demonstrates that if we query for the same row we get the same reference (entity type)
        private static void DemonstrateIdentityMap()
        {
            NorthwindContext session = new NorthwindContext("Server=.;Database=Northwind;Trusted_Connection=True;", Mapping.GetMapping());
            session.Log = Console.Out;

            var customer = session.Customers.Single(c => c.CustomerID == "CONSH");

            IList<Customer> customers =
                    (from c in session.Customers
		            where c.City == "London"
                    select c).ToList();

            var matchingCustomer = customers.Single(c => c.CustomerID == "CONSH");

            ObjectDumper.Write(customer == matchingCustomer);
        }
        #endregion

        #region 6: We can make an ad-hoc join, not just those that we specify with relationships
        private static void AdhocJoins()
        {
            NorthwindContext session = new NorthwindContext("Server=.;Database=Northwind;Trusted_Connection=True;", Mapping.GetMapping());
            session.Log = Console.Out;

            var query =
                  from c in session.Customers
		                  join s in session.Suppliers on c.Country equals s.Country
		                  select new {CustgomerName = c.CompanyName, SupplierName = s.CompanyName};


            ObjectDumper.Write(query);
        }

        #endregion

        #region 7: We can use into to support heirachical queries
        private static void WorkingWithHeirachicalData()
        {
            NorthwindContext session = new NorthwindContext("Server=.;Database=Northwind;Trusted_Connection=True;", Mapping.GetMapping());
            session.Log = Console.Out;

            var query =
              from c in session.Customers
		              join s in session.Suppliers on c.Country equals s.Country into sups
		              select new {CustgomerName = c.CompanyName,
		                  SupplierName = from s in sups select s.CompanyName};

            ObjectDumper.Write(query, 1);
        }
        #endregion

        #region 8: Execution - issuing the SQL - is deferred until we execute the query
        private static void DeferredExecution()
        {
            NorthwindContext session = new NorthwindContext("Server=.;Database=Northwind;Trusted_Connection=True;", Mapping.GetMapping());
            session.Log = Console.Out;

            var query =
                 from c in session.Customers
		                 where c.City == "London"
		                 from o in c.Orders
		                 select c;

            var query2 =
                        from c in query
		                        select new {c.CompanyName, c.Phone};

            ObjectDumper.Write(query2);
        }

        #endregion

        #region 9: Show the Lazy Loading of child collections - but note that DataShape changes this
        private static void LazyLoading()
        {
            NorthwindContext session = new NorthwindContext("Server=.;Database=Northwind;Trusted_Connection=True;", Mapping.GetMapping());
            session.Log = Console.Out;

            var query =
            from c in session.Customers
		             where c.City == "London"
		             from o in c.Orders
		             select c;

            foreach (var c in query)
                foreach (var o in c.Orders)
                    Console.WriteLine(o.OrderID);
        }
        #endregion

        #region 10: Shows Add and Delete; same thing is true of update; submitchanges to flush the session otherwise in cache
        private static void AddAndDeleteRows()
        {
 
            NorthwindContext session = new NorthwindContext("Server=.;Database=Northwind;Trusted_Connection=True;", Mapping.GetMapping());
            session.Log = Console.Out;

           Customer cust = new Customer();
            cust.CustomerID = "AAAA";
            cust.CompanyName = "Cigars Are Us";
            cust.ContactName = "Fidel";
            cust.City = "London";
            cust.Country = "UK";

            session.Customers.Add(cust);
            session.SubmitChanges();

            ObjectDumper.Write(session.Customers.Where(c => c.City == "London"));


            //Now lets kill the added record so we can re-run the query; alternative would be TransactionScope()
            Customer customer =
                (from c in session.Customers
                 where c.CustomerID == "AAAA"
                 select c).Single();

            session.Customers.Remove(customer);

            session.SubmitChanges();

            ObjectDumper.Write(session.Customers.Where(c => c.City == "London"));
        }
        #endregion

        #region 11: Update a customer
        private static void Update()
        {
            NorthwindContext session = new NorthwindContext("Server=.;Database=Northwind;Trusted_Connection=True;", Mapping.GetMapping());
            session.Log = Console.Out;

            Customer cust = new Customer();
            cust.CustomerID = "AAAA";
            cust.CompanyName = "Cigars Are Us";
            cust.ContactName = "Fidel";
            cust.City = "London";
            cust.Country = "UK";

            session.Customers.Add(cust);
            session.SubmitChanges();

            ObjectDumper.Write(session.Customers.Where(c => c.City == "London"));

            cust.CompanyName = "Virgin Rolled";
            session.SubmitChanges();
            ObjectDumper.Write(session.Customers.Where(c => c.City == "London"));

            Customer customer =
                (from c in session.Customers
                 where c.CustomerID == "AAAA"
                 select c).Single();

            //Now lets kill the added record so we can re-run the query; alternative would be TransactionScope()
            session.Customers.Remove(customer);

            session.SubmitChanges();

            ObjectDumper.Write(session.Customers.Where(c => c.City == "London"));
        }
        #endregion

    }
}
