using System;
using System.Collections.Generic;
using System.Text;
using FullGen;
using System.Query;
using System.Xml.XLinq;
using System.Data.DLinq;
using System.IO;
using System.Reflection;
using System.Transactions;


namespace LINQToSQL
{
    public static class FullGenDemo
    {
        public class CustomerPhone
        {
            public string CompanyName;
            public string Phone;
        }

        public static void Demo()
        {
            //CallAStoredProcedure();
            //PassThroughSQL();
            UserGeneratedUpdateStatement();
        }

        #region 1: We can call a stored procedure using LINQ syntax
        private static void CallAStoredProcedure()
        {
            FullGen.Northwind session = new FullGen.Northwind("Server=.;Database=Northwind;Trusted_Connection=True;");
            session.Log = Console.Out;

            var query =
            from c in session.CustomersByCity("London")
		            where c.CustomerID == "AROUT"
		            select c;

            ObjectDumper.Write(query);
        }
        #endregion

        #region 2: We can also pass SQL through to the DB if we need to
        private static void PassThroughSQL()
        {
            FullGen.Northwind session = new FullGen.Northwind("Server=.;Database=Northwind;Trusted_Connection=True;");
            session.Log = Console.Out;

            var query = session.ExecuteQuery<CustomerPhone>("SELECT CompanyName as Name, Phone FROM Customers Where City = 'London'");


            ObjectDumper.Write(query);
        }
        #endregion

        #region 3: Shows how we can add our own stored procedure call instead of using dynamic sql
        private static void UserGeneratedUpdateStatement()
        {
            FullGen.Northwind session = new FullGen.Northwind("Server=.;Database=Northwind;Trusted_Connection=True;");
            session.Log = Console.Out;

            var query =
                from p in session.Products
                    where p.SupplierID == 3
                    orderby p.UnitsInStock
                    select p;

            FullGen.Product sauce = query.First();

            Console.WriteLine("\nID={0}\tStock={1}\tName={2}\n",
                sauce.ProductID, sauce.UnitsInStock, sauce.ProductName);

            sauce.UnitsInStock += 10;

            using (new TransactionScope())
            {
                session.SubmitChanges();
            }
        }
        #endregion
    }
}
