﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Linq;
using System.Xml.Linq;
using LinqToXMLQueries;

namespace XLINQQueries
{
    class Program
    {
        static void Main(string[] args)
        {
            //ParseXMLTreeFromString();
            //FunctionalConstructionAndQuery();
            XLinqAndDLinq();
        }

        #region 1: Shows how to load an XElement by parsing a string
        private static void ParseXMLTreeFromString()
        {
            XElement contacts = LoadContactsFromString();
            Console.WriteLine(contacts.ToString());
        }
        #endregion

        #region 2: Uses a query with functional constuction to create a document
        /// <summary>
        /// Note that this is the basis of how to doa transform and allows us to use functional construction
        /// as an alternative to XSLT
        /// </summary>
        private static void FunctionalConstructionAndQuery()
        {
            var persons = new []
            {
	            new Person
                {
		            Name = "Patrick Hines",
		            PhoneNumbers = new[] { "206-555-0144", "425-555-0145" }
	            },
	            new Person
                {
		            Name = "Gretchen Rivas",
		            PhoneNumbers = new[] { "206-555-0163" }
	            }
            };

            XElement contacts =
			            new XElement("contacts",
                            from p in persons
				                    select new XElement("contact",
					                    new XElement("name", p.Name),
					                    from ph in p.PhoneNumbers
					                    select new XElement("phone", ph)
				            )
                );

            Console.WriteLine(contacts);
        }
        #endregion

        #region 3: Combines XLinq with DLinq to populate an XML document from a DB.
        private static void XLinqAndDLinq()
        {
            NorthwindDataContext session = new NorthwindDataContext();

            XElement xml =
		        new XElement("Results",
                    from c in session.Customers
		                    where c.City == "London"
		                    select new XElement("Contact", c.ContactName,
		                        new XElement("Address", c.Address),
		                        new XElement("City", c.City),
		                        new XElement("PostCode", c.PostalCode),
		                        new XElement("Country", c.Country)
		                    )
                );

            Console.WriteLine(xml.ToString());
        }
        #endregion
        
        #region Load methods
          private static XElement LoadContactsFromString()
        {
            XElement contacts = XElement.Parse(
                                        @"<contacts>
						            <contact>
							            <name>Patrick Hines</name>
							            <phone type=""home"">206-555-0144</phone>
							            <phone type=""work"">425-555-0145</phone>
							            <address>
								            <street1>123 Main St</street1>
								            <city>Mercer Island</city>
								            <state>WA</state>
								            <postal>68042</postal>
							            </address>
							            <netWorth>10</netWorth>
						            </contact>
						            <contact>
							            <name>Gretchen Rivas</name>
							            <phone type=""mobile"">206-555-0163</phone>
							            <address>
								            <street1>123 Main St</street1>
								            <city>Mercer Island</city>
								            <state>WA</state>
								            <postal>68042</postal>
							            </address>
							            <netWorth>11</netWorth>
						            </contact>
						            <contact>
							            <name>Scott MacDonald</name>
							            <phone type=""home"">925-555-0134</phone>
							            <phone type=""mobile"">425-555-0177</phone>
							            <address>
								            <street1>345 Stewart St</street1>
								            <city>Chatsworth</city>
								            <state>CA</state>
								            <postal>91746</postal>
							            </address>
							            <netWorth>500000</netWorth>
						            </contact>
					            </contacts>");
            return contacts;
        }

        private static XElement LoadContacts()
        {
            return new XElement("contacts",
                    new XElement("contact",
                        new XElement("name", "Patrick Hines"),
                        new XElement("phone", "206-555-0144"),
                        new XElement("address",
                            new XElement("street1", "123 Main St"),
                            new XElement("city", "Mercer Island"),
                            new XElement("state", "WA"),
                            new XElement("postal", "68042")
                    )
                )
            );
        }
        #endregion
    }
}
